using System;
using Views;

namespace University {

  /// <summary>
  /// The University Access Control System    Bishop & Horspool  2002
  /// ====================================
  /// Registers details of staff and students then 
  /// validates who has access for the next year.
  /// Illustrates defining and using interfaces.
  /// </summary>

  interface IAccess {
    bool Valid();
  }

  class AccessControl {

    Views.Form f;

    void SetUpForm() {
      try {
        f = new Views.Form(@"<Form>
        <vertical>
        <Label Text='Welcome to Access Control Update'
              Width=350 Font=SansSerifBold14 ForeColor=red/>
        <CheckBox Name=Staff Checked=1/>
        <CheckBox Name=Student/>
        <horizontal>
          <Label Text='Name' />
          <TextBox Name=Name Text='Guest'/>
          <Label Text='for staff'/>
        </horizontal>
        <horizontal>
          <Label Text='Number' />
          <TextBox Name=Number Text='20031111'/>
          <Label Text='for students'/>
        </horizontal>
        <horizontal>
          <Button Name = Enter/>
          <Button Name = Appoint/>
          <Button Name = 'End of Year'/>
        </horizontal>
        <ListBox Name=List Width=350/>
        </vertical>
        </Form>" );
      } catch( Exception e ) {
        Console.WriteLine(
          "Exception caught, message follows:\n{0}", e.Message);
      }
    }

    void Go() {
      SetUpForm();
      string c;
      for (;;) {
        c = f.GetControl();
        if (c==null) break;
        ActionPerformed(c);
      }
      f.Dispose();
    }

    int kind;
    int number;
    string name;
    IAccess[] register = new IAccess[100];
    int count;

    void HandleAppointments() {
      number = int.Parse(f.GetText("Number"));
      bool found = false;
      int i;
      Student s;
      for (i = 0; i<count; i++) {
        if (register[i] is Student) {
          s = (Student) register[i];
          if (s.Number == number) {
            found = true;
            break;
          }
        }
      }
      if (found) {
        s=(Student) register[i];
        s.Appoint();
        f.PutText("List",s+" appointed: years now "
                +s.TutorYears);
      } else
        f.PutText("List","No such student\n");
    }

    void EnterPerson() {
      kind = f.GetValue("Student");
      switch (kind) {
        case 1 : {
          number = int.Parse(f.GetText("Number"));
          register[count] = new Student(number);
          break;
        }
        case 0 : {
          name = f.GetText("Name");
          register[count] = new Staff(name);
          break;
        }
      }
      f.PutText("List",register[count].ToString());
      count++;
    }

    void ActionPerformed (string c) {
      switch (c) {
        case "Student" : {
          f.PutValue("Staff",0);
          break;
        }
        case "Staff" : {
          f.PutValue("Student",0);
          break;
        }
        case "Name": case "Number" : {
          break;
        }
        case "Enter" : {
          EnterPerson();
          break;
        } 
        case "Appoint" : {
          HandleAppointments();
          break;
        }
        case "End of Year" : {
          UpdateAccess();
          break;
        }
      } // case c
    } // ActionPerformed

    void UpdateAccess () {
      Array.Sort(register,0,count);
      f.PutText("List", "People granted access for the new year");
      for (int i = 0; i<count; i++) {
        if (register[i].Valid()) 
          f.PutText("List",register[i]+"         Access Valid\n");
        else
          f.PutText("List",register[i]+"         No Access\n");
      } 
    }

    static void Main() {
      new AccessControl().Go();
    }
  }

  class Student : IAccess, IComparable {
    int number;
    int tutorYears;
    public Student(int n) {
      number = n;
      tutorYears = 0;
    }
    public void Appoint () {
      tutorYears++;
    }
    public int Number {
      get {return number;}
    }
    public int TutorYears {
      get {return tutorYears;}
    }
    public bool Valid() {
      return (tutorYears > 0);
    }
    public override string ToString () {
      if (tutorYears == 0) return "Student "
        +number+"("+tutorYears+")";
      else return "Tutor "+number+"("+tutorYears+")";
    }
    public int CompareTo(object o) {
      if (o is Student) 
        return (number.CompareTo(((Student) o).number));
      else // o is Staff; Staff always rate before students
        return 1;
    }
  }

  public class Staff : IAccess, IComparable {
    string name;
    int startDate;

    public Staff(string n) {
      name = n;
      startDate = DateTime.Now.Year;
    }
    public string Name {
      get {return name;}
    }
    public int StartDate {
      get {return startDate;}
    }
    public bool Valid() {
      return true;
    }
    public override string ToString () {
      return "Staff "+name;
    }
    public int CompareTo(object o) {
      if (o is Staff) 
        return (name.CompareTo(((Staff) o).name));
      // o is a Student; Staff always rate before students
      return -1;
    }
  }
}